package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.apiservice.serviceimpl.base.BaseBusiSyncMdmDataService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MdmEnumUtil;
import cn.estsh.i3plus.pojo.mdm.bean.base.MdmSyncData;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;

import static cn.estsh.i3plus.platform.common.util.MdmConstWords.REDIS_MDM_SYNC_FAIL;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-05-28 16:54
 * @Modify:
 **/
public abstract class BaseMdmDataSyncQueueReceiver {

    private static final Logger LOGGER = LoggerFactory.getLogger(BaseMdmDataSyncQueueReceiver.class);

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Resource(name = CommonConstWords.IMPP_REDIS_MDM)
    private ImppRedis mdmRedis;

    private static final HashMap<String, Class<BaseBusiSyncMdmDataService>> masterDateRefMap = new HashMap<>();
    private static final HashMap<String, Class<BaseBusiSyncMdmDataService>> busiDateRefMap = new HashMap<>();
    private static final HashMap<Class<BaseBusiSyncMdmDataService>, BaseBusiSyncMdmDataService> syncServiceMap = new HashMap<>();

    public void putMdmDataRef(Class syncService, Class mdmMasterClass, Class mdmBusiClass) {
        masterDateRefMap.put(mdmMasterClass.getName(), syncService);
        busiDateRefMap.put(mdmBusiClass.getName(), syncService);
    }

    /**
     * 获取同步服务
     *
     * @param syncData
     * @return
     */
    private BaseBusiSyncMdmDataService getSyncService(MdmSyncData syncData) {
        Class<BaseBusiSyncMdmDataService> syncServiceClass = null;

        // 根据MdmClass查找对应关系
        if (CommonEnumUtil.TRUE_OR_FALSE.valueOfBoolean(syncData.getIsMasterDataVal())) {
            syncServiceClass = masterDateRefMap.get(syncData.getSyncDataClass());
        } else {
            syncServiceClass = busiDateRefMap.get(syncData.getSyncDataClass());
        }

        if (syncServiceClass == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MDM.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("【MDM同步关系未维护】MdmClass %s", syncData.getSyncDataClass())
                    .setErrorSolution("维护Mdm同步关系")
                    .build();
        }

        // 根据同步服务Class获取服务对象
        BaseBusiSyncMdmDataService busiSyncMdmDataService = syncServiceMap.get(syncServiceClass);
        if (busiSyncMdmDataService == null) {
            busiSyncMdmDataService = SpringContextsUtil.getBean(syncServiceClass);
            syncServiceMap.put(syncServiceClass, busiSyncMdmDataService);
        }
        if (busiSyncMdmDataService == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MDM.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("【MDM同步服务为找到】SyncServiceClass %s", syncServiceClass)
                    .build();
        }

        return busiSyncMdmDataService;
    }

    /**
     * Mdm主数据同步
     *
     * @param syncDataStr 消息
     * @param channel     通道
     * @param message     消息
     */
    public void processSyncMasterData(String syncDataStr, Channel channel, Message message) {
        MdmSyncData syncData = null;
        try {
            LOGGER.info("【MQ-QUEUE_MDM_SYNC_MASTER_DATA_CORE】数据接收成功：{}", syncDataStr);
            syncData = JsonUtilTool.decode(syncDataStr, MdmSyncData.class);
            if (syncData == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("同步数据不存在")
                        .build();
            }

            // 同步数据类型
            BaseBusiSyncMdmDataService busiSyncMdmDataService = getSyncService(syncData);

            boolean isMaster = CommonEnumUtil.TRUE_OR_FALSE.valueOfBoolean(syncData.getIsMasterDataVal());

            // 判断操作类型与同步数据类型调用对应的方法
            if (syncData.getSyncTypeVal() == MdmEnumUtil.MDM_SYNC_TYPE.INSERT.getValue()) {
                if (isMaster) {
                    busiSyncMdmDataService.masterInsertSync(syncData);
                } else {
                    busiSyncMdmDataService.busiInsertSync(syncData);
                }
            } else if (syncData.getSyncTypeVal() == MdmEnumUtil.MDM_SYNC_TYPE.UPDATE.getValue()) {
                if (isMaster) {
                    busiSyncMdmDataService.masterUpdateSync(syncData);
                } else {
                    busiSyncMdmDataService.busiUpdateSync(syncData);
                }
            } else if (syncData.getSyncTypeVal() == MdmEnumUtil.MDM_SYNC_TYPE.DELETE.getValue()) {
                if (isMaster) {
                    busiSyncMdmDataService.masterDeleteSync(syncData);
                } else {
                    busiSyncMdmDataService.busiDeleteSync(syncData);
                }
            }

            //信息已处理
            LOGGER.info("【MQ-QUEUE_MDM_SYNC_MASTER_DATA_CORE】数据处理成功：{}", syncDataStr);
        } catch (Exception e) {
            LOGGER.error("【MQ-QUEUE_MDM_SYNC_MASTER_DATA_CORE】处理出错：{}", syncData, e);
            if (syncData != null) {
                // 保存异常信息和堆栈数据
                syncData.setSyncFailDateTime(TimeTool.getNowTime(true));
                syncData.setSyncFailMessage(ExceptionUtils.getStackTrace(e));

                //放入MDM 缓存中
                mdmRedis.putObject(getFailRedisKey(syncData), syncData, -1);
            }
        } finally {
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }

    private String getFailRedisKey(MdmSyncData syncData) {
        return REDIS_MDM_SYNC_FAIL + "-" + syncData.getSyncSoftType() + ":" + syncData.getSyncDataClassSimpleName() + ":" + syncData.getSyncFailDateTime() + "-" + System.currentTimeMillis();
    }

    public abstract void syncMasterData(String syncDataStr, Channel channel, Message message);

}
